@@ -24,12 +24,17 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include <math.h>
 #include <sys/types.h>
 #include <stdio.h>
-#include <gsl/gsl_sort.h>
 #include "base/Optionpk.h"
 #include "base/Vector2d.h"
 #include "algorithms/Filter.h"
 #include "fileclasses/FileReaderAscii.h"
 
+extern "C" {
+#include <gsl/gsl_sort.h>
+}
+
+using namespace std;
+
 /*------------------
   Main procedure
   ----------------*/
@@ -37,10 +42,10 @@ int main(int argc,char **argv) {
   Optionpk<std::string> input_opt("i","input","input ASCII file");
   Optionpk<std::string> output_opt("o", "output", "Output ASCII file");
   Optionpk<int> inputCols_opt("ic", "inputCols", "input columns (e.g., for three dimensional input data in first three columns use: -ic 0 -ic 1 -ic 2"); 
-  Optionpk<std::string> method_opt("f", "filter", "filter function (to be implemented: dwtForward, dwtInverse,dwtQuantize)");
+  Optionpk<std::string> method_opt("f", "filter", "filter function (to be implemented: dwt, dwti,dwt_cut)");
   Optionpk<std::string> wavelet_type_opt("wt", "wavelet", "wavelet type: daubechies,daubechies_centered, haar, haar_centered, bspline, bspline_centered", "daubechies");
   Optionpk<int> family_opt("wf", "family", "wavelet family (vanishing moment, see also http://www.gnu.org/software/gsl/manual/html_node/DWT-Initialization.html)", 4);
-  Optionpk<double> threshold_opt("qt", "threshold", "Quantize threshold value", 0);
+  Optionpk<double> threshold_opt("cut", "cut", "threshold to cut dwt coefficients. Use 0 to keep all.", 0);
   Optionpk<int> dimZ_opt("dz", "dz", "filter kernel size in z (band or spectral dimension), must be odd (example: 3).. Set dz>0 if 1-D filter must be used in band domain");
   Optionpk<double> tapZ_opt("tapz", "tapz", "taps used for spectral filtering");
   Optionpk<double> fwhm_opt("fwhm", "fwhm", "list of full width half to apply spectral filtering (-fwhm band1 -fwhm band2 ...)");
@@ -93,7 +98,8 @@ int main(int argc,char **argv) {
     std::cout << "wavelengthIn.size(): " << wavelengthIn.size() << std::endl;
     std::cout << "inputData[0].size(): " << inputData[0].size() << std::endl;
   }
-  assert(wavelengthIn.size()==inputData[0].size());
+  if(wavelengthIn.size())
+    assert(wavelengthIn.size()==inputData[0].size());
   asciiReader.close();
   filter::Filter filter1d;
   if(fwhm_opt.size()){
@@ -168,59 +174,34 @@ int main(int argc,char **argv) {
   }
   
   if(method_opt.size()){
-    switch(filter::Filter::getFilterType(method_opt[0])){
-    case(filter::smooth):
-      assert(dimZ_opt.size());
-      if(tapZ_opt.empty()){
-        tapZ_opt.resize(dimZ_opt[0]);
-        for(int itap=0;itap<dimZ_opt[0];++itap)
-          tapZ_opt[itap]=1.0/dimZ_opt[0];
-      }
-      filter1d.setTaps(tapZ_opt);
-      case(filter::dwtQuantize)://deliberate fall through
-      wavelengthOut=wavelengthIn;
-      break;
-    }
+    wavelengthOut=wavelengthIn;
     for(int icol=0;icol<inputCols_opt.size();++icol){
       switch(filter::Filter::getFilterType(method_opt[0])){
+      case(filter::dwt):
+        filter1d.dwtForward(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
+        break;
+      case(filter::dwti):
+        filter1d.dwtInverse(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
+        break;
+      case(filter::dwt_cut):
+	filter1d.dwtCut(filteredData[icol],wavelet_type_opt[0],family_opt[0],threshold_opt[0]);
+        break;
       case(filter::smooth):
-        filter1d.doit(inputData[icol],filteredData[icol]);
-        break;
-      case(filter::dwtForward):
-        filter1d.dwtForward(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
-        break;
-      case(filter::dwtInverse):
-        filter1d.dwtInverse(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
-        break;
-      case(filter::dwtQuantize):{
-        int origSize=filteredData[icol].size();
-        filter1d.dwtForward(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
-        std::vector<double> abscoeff(filteredData[icol].size());
-        for(int iband=0;iband<filteredData[icol].size();++iband){
-          abscoeff[iband]=fabs(filteredData[icol][iband]);
-          if(threshold_opt[0]<0){//absolute threshold
-            if(abscoeff[iband]<-threshold_opt[0])
-              filteredData[icol][iband]=0;
-          }
-          // if(fabs(filteredData[icol][iband])<threshold_opt[0])
-          //   filteredData[icol][iband]=0;
-        }
-        if(threshold_opt[0]>0){//percentual threshold
-          int nsize=abscoeff.size();
-          size_t* p=new size_t[nsize];
-          int nc=threshold_opt[0]/100.0*nsize;
-          gsl_sort_index(p,&(abscoeff[0]),1,nsize);
-          for(int i=0;(i+nc)<nsize;i++)
-            filteredData[icol][p[i]]=0;
-        }
-        filter1d.dwtInverse(filteredData[icol],wavelet_type_opt[0],family_opt[0]);
-        //remove extended samples
-        filteredData[icol].erase(filteredData[icol].begin()+origSize,filteredData[icol].end());
-        break;
-      }
+	if(tapZ_opt.size()){
+	  filter1d.setTaps(tapZ_opt);
+	  filter1d.filter(inputData[icol],filteredData[icol]);
+	}
+	else{
+	  assert(dimZ_opt.size());
+	  filter1d.smooth(inputData[icol],filteredData[icol],dimZ_opt[0]);
+	}
+	break;
       default:
-        if(verbose_opt[0])
-          std::cout << "method to be implemented" << std::endl;
+	assert(tapZ_opt.size());
+        filter1d.filter(inputData[icol],filteredData[icol]);
+        // if(verbose_opt[0])
+        //   std::cout << "method to be implemented" << std::endl;
+	// exit(1);
         break;
       }
     }
@@ -256,16 +237,14 @@ int main(int argc,char **argv) {
     int nband=0;
     if(method_opt.size()){
       switch(filter::Filter::getFilterType(method_opt[0])){
-      case(filter::dwtForward):
+      case(filter::dwt):
         nband=filteredData[0].size();
         break;
-      case(filter::dwtInverse):
+      case(filter::dwti):
         nband=filteredData[0].size();
         break;
-      case(filter::dwtQuantize):
-        nband=wavelengthOut.size();
-        assert(wavelengthOut.size()==nband);
-        assert(filteredData[0].size()==nband);
+      case(filter::dwt_cut):
+        nband=filteredData[0].size();
         break;
       default:
         nband=wavelengthOut.size();
@@ -282,13 +261,13 @@ int main(int argc,char **argv) {
       if(!output_opt.empty()){
         if(wavelengthOut.size())
           outputStream << wavelengthOut[iband] << " ";
-        else
+        else if(wavelengthIn_opt.size())
           outputStream << iband << " ";
       }
       else{
         if(wavelengthOut.size())
           std::cout << wavelengthOut[iband] << " ";
-        else
+        else if(wavelengthIn_opt.size())
           std::cout << iband << " ";
       }
       for(int icol=0;icol<inputCols_opt.size();++icol){
