@@ -32,79 +32,91 @@ ImgWriterOgr::~ImgWriterOgr(void)
 {
 }
 
-ImgWriterOgr::ImgWriterOgr(const string& filename)
+ImgWriterOgr::ImgWriterOgr(const std::string& filename, const std::string& imageType)
 {
-  open(filename);
+  open(filename,imageType);
 }
 
-ImgWriterOgr::ImgWriterOgr(const string& filename, ImgReaderOgr& imgReaderOgr)
+ImgWriterOgr::ImgWriterOgr(const std::string& filename, ImgReaderOgr& imgReaderOgr)
 {
   m_filename=filename;
   setCodec(imgReaderOgr.getDriver());
-  createLayer(filename,imgReaderOgr.getProjection(),imgReaderOgr.getGeometryType(),NULL);
-  copyFields(imgReaderOgr);
+  int nlayer=imgReaderOgr.getDataSource()->GetLayerCount();
+  for(int ilayer=0;ilayer<nlayer;++ilayer){
+    std::string layername = imgReaderOgr.getLayer(ilayer)->GetName();
+    createLayer(layername,imgReaderOgr.getProjection(),imgReaderOgr.getGeometryType(),NULL);
+    copyFields(imgReaderOgr,ilayer);
+  }
 }
 
-ImgWriterOgr::ImgWriterOgr(const string& filename, ImgReaderOgr& imgReaderOgr, bool copyData)
+ImgWriterOgr::ImgWriterOgr(const std::string& filename, ImgReaderOgr& imgReaderOgr, bool copyData)
 {
   CPLErrorReset();
   m_filename=filename;
   setCodec(imgReaderOgr.getDriver());
-  createLayer(filename,imgReaderOgr.getProjection(),imgReaderOgr.getGeometryType(),NULL);
-  copyFields(imgReaderOgr);
-  if(copyData){
-    OGRFeature *poFeature;
-    while(true){// (poFeature = imgReaderOgr.getLayer()->GetNextFeature()) != NULL ){
-      poFeature = imgReaderOgr.getLayer()->GetNextFeature();
-      if( poFeature == NULL )
-        break;
-      OGRFeature *poDstFeature = NULL;
+  int nlayer=imgReaderOgr.getDataSource()->GetLayerCount();
+  for(int ilayer=0;ilayer<nlayer;++ilayer){
+    std::string layername = imgReaderOgr.getLayer(ilayer)->GetName();
+    createLayer(layername,imgReaderOgr.getProjection(),imgReaderOgr.getGeometryType(),NULL);
+    copyFields(imgReaderOgr,ilayer);
+    if(copyData){
+      OGRFeature *poFeature;
+      while(true){// (poFeature = imgReaderOgr.getLayer()->GetNextFeature()) != NULL ){
+	poFeature = imgReaderOgr.getLayer(ilayer)->GetNextFeature();
+	if( poFeature == NULL )
+	  break;
+	OGRFeature *poDstFeature = NULL;
 
-      poDstFeature=createFeature();
-//       poDstFeature = OGRFeature::CreateFeature(m_datasource->GetLayer(m_datasource->GetLayerCount()-1)->GetLayerDefn());
-      if( poDstFeature->SetFrom( poFeature, TRUE ) != OGRERR_NONE ){
-	const char* fmt;
-	string errorString="Unable to translate feature %d from layer %s.\n";
-	fmt=errorString.c_str();
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  fmt,
-                  poFeature->GetFID(), getLayerName().c_str() );
-        // CPLError( CE_Failure, CPLE_AppDefined,
-        //           "Unable to translate feature %d from layer %s.\n",
-        //           poFeature->GetFID(), getLayerName().c_str() );
+	poDstFeature=createFeature(ilayer);
+	//todo: check here if SetFrom works (experienced segmentation fault)
+	if( poDstFeature->SetFrom( poFeature, TRUE ) != OGRERR_NONE ){
+	  const char* fmt;
+	  std::string errorString="Unable to translate feature %d from layer %s.\n";
+	  fmt=errorString.c_str();
+	  CPLError( CE_Failure, CPLE_AppDefined,
+		    fmt,
+		    poFeature->GetFID(), getLayerName().c_str() );
+	  // CPLError( CE_Failure, CPLE_AppDefined,
+	  //           "Unable to translate feature %d from layer %s.\n",
+	  //           poFeature->GetFID(), getLayerName().c_str() );
             
-        OGRFeature::DestroyFeature( poFeature );
-        OGRFeature::DestroyFeature( poDstFeature );
-      }
-      poDstFeature->SetFID( poFeature->GetFID() );
-      OGRFeature::DestroyFeature( poFeature );
+	  OGRFeature::DestroyFeature( poFeature );
+	  OGRFeature::DestroyFeature( poDstFeature );
+	}
+	poDstFeature->SetFID( poFeature->GetFID() );
+	OGRFeature::DestroyFeature( poFeature );
 
-      CPLErrorReset();
-      if(createFeature( poDstFeature ) != OGRERR_NONE){
-	const char* fmt;
-	string errorString="Unable to translate feature %d from layer %s.\n";
-	fmt=errorString.c_str();
-        CPLError( CE_Failure, CPLE_AppDefined,
-		  fmt,
-                  poFeature->GetFID(), getLayerName().c_str() );
-        OGRFeature::DestroyFeature( poDstFeature );
+	CPLErrorReset();
+	if(createFeature( poDstFeature,ilayer ) != OGRERR_NONE){
+	  const char* fmt;
+	  std::string errorString="Unable to translate feature %d from layer %s.\n";
+	  fmt=errorString.c_str();
+	  CPLError( CE_Failure, CPLE_AppDefined,
+		    fmt,
+		    poFeature->GetFID(), getLayerName().c_str() );
+	  OGRFeature::DestroyFeature( poDstFeature );
+	}
+	OGRFeature::DestroyFeature( poDstFeature );
       }
-      OGRFeature::DestroyFeature( poDstFeature );
     }
   }
 }
 
 //---------------------------------------------------------------------------
 
-void ImgWriterOgr::open(const string& filename, ImgReaderOgr& imageReader)
+void ImgWriterOgr::open(const std::string& filename, ImgReaderOgr& imgReaderOgr)
 {
   m_filename=filename;
-  setCodec(imageReader.getDriver());
-  createLayer(filename,imageReader.getProjection(),imageReader.getGeometryType(),NULL);
-  copyFields(imageReader);
+  setCodec(imgReaderOgr.getDriver());
+  int nlayer=imgReaderOgr.getDataSource()->GetLayerCount();
+  for(int ilayer=0;ilayer<nlayer;++ilayer){
+    std::string layername = imgReaderOgr.getLayer(ilayer)->GetName();
+    createLayer(layername,imgReaderOgr.getProjection(),imgReaderOgr.getGeometryType(),NULL);
+    copyFields(imgReaderOgr,ilayer);
+  }
 }
 
-void ImgWriterOgr::open(const string& filename, const string& imageType)
+void ImgWriterOgr::open(const std::string& filename, const std::string& imageType)
 {
   m_filename = filename;
   setCodec(imageType);
@@ -117,20 +129,20 @@ void ImgWriterOgr::close(void)
 }
 
 //---------------------------------------------------------------------------
-void ImgWriterOgr::setCodec(const string& imageType){
+void ImgWriterOgr::setCodec(const std::string& imageType){
   //register the drivers
   OGRRegisterAll();
   //fetch the shape file driver
   OGRSFDriver *poDriver;
   poDriver = OGRSFDriverRegistrar::GetRegistrar()->GetDriverByName(imageType.c_str());
   if( poDriver == NULL ){
-    string errorString="FileOpenError";
+    std::string errorString="FileOpenError";
     throw(errorString);
   }
   //create the data source
   m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
   if(m_datasource==NULL){
-    string errorString="Creation of output file failed";
+    std::string errorString="Creation of output file failed";
     throw(errorString);
   }
 }
@@ -138,57 +150,63 @@ void ImgWriterOgr::setCodec(const string& imageType){
 void ImgWriterOgr::setCodec(OGRSFDriver *poDriver){
   OGRRegisterAll();
   if( poDriver == NULL ){
-    string errorString="FileOpenError";
+    std::string errorString="FileOpenError";
     throw(errorString);
   }
   //create the data source
   m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
   if(m_datasource==NULL){
-    string errorString="Creation of output file failed";
+    std::string errorString="Creation of output file failed";
     throw(errorString);
   }
 }
 
-// OGRLayer* ImgWriterOgr::copyLayer(OGRLayer* poSrcLayer, const string& layername, char** papszOptions)
+// OGRLayer* ImgWriterOgr::copyLayer(OGRLayer* poSrcLayer, const std::string& layername, char** papszOptions)
 // {
 //   return(m_datasource->CopyLayer(poSrcLayer, layername.c_str(),papszOptions));
 // }
 
-OGRLayer* ImgWriterOgr::createLayer(const string& layername, const string& theProjection, const OGRwkbGeometryType& eGType, char** papszOptions)
+OGRLayer* ImgWriterOgr::createLayer(const std::string& layername, const std::string& theProjection, const OGRwkbGeometryType& eGType, char** papszOptions)
 {
   if( !m_datasource->TestCapability( ODsCCreateLayer ) ){
-    string errorString="Test capability to create layer failed";
+    std::string errorString="Test capability to create layer failed";
     throw(errorString);
   }
   //papszOptions = CSLSetNameValue( papszOptions, "DIM", "1" );
   //if points: use wkbPoint
   //if no constraints on the types geometry to be written: use wkbUnknown 
   OGRLayer* poLayer;
+  OGRSpatialReference oSRS;
+
   if(theProjection!=""){
-    if(theProjection.find("EPSPG:")!=string::npos){
-      int epsg_code=atoi(theProjection.substr(theProjection.find_first_not_of("EPSG:")).c_str());
-      OGRSpatialReference oSRS;
-      oSRS.importFromEPSG(epsg_code);
-      poLayer=m_datasource->CreateLayer( layername.c_str(), &oSRS, eGType,papszOptions );
-    }
-    else{
-      OGRSpatialReference oSRS(theProjection.c_str());
-      poLayer=m_datasource->CreateLayer( layername.c_str(), &oSRS, eGType,papszOptions );
-    }
+    oSRS.SetFromUserInput(theProjection.c_str());
+    poLayer=m_datasource->CreateLayer( layername.c_str(), &oSRS, eGType,papszOptions );
+    //   if(theProjection.find("EPSPG:")!=std::string::npos){
+    //     int epsg_code=atoi(theProjection.substr(theProjection.find_first_not_of("EPSG:")).c_str());
+    //     OGRSpatialReference oSRS;
+    //     oSRS.importFromEPSG(epsg_code);
+    //     poLayer=m_datasource->CreateLayer( layername.c_str(), &oSRS, eGType,papszOptions );
+    //   }
+    //   else{
+    //     OGRSpatialReference oSRS(theProjection.c_str());
+    //     poLayer=m_datasource->CreateLayer( layername.c_str(), &oSRS, eGType,papszOptions );
+    //   }
+    // }
+    // oSRS.importFromProj4(theProjection);
   }
   else
     poLayer=m_datasource->CreateLayer( layername.c_str(), NULL, eGType,papszOptions );
   //check if destroy is needed?!
   CSLDestroy( papszOptions );
   if( poLayer == NULL ){
-    string errorstring="Layer creation failed";
+    std::string errorstring="Layer creation failed";
     throw(errorstring);
   }
-  OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
+  // OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
   return poLayer;
 }
 
-void ImgWriterOgr::createField(const string& fieldname, const OGRFieldType& fieldType, int theLayer)
+void ImgWriterOgr::createField(const std::string& fieldname, const OGRFieldType& fieldType, int theLayer)
 {
   OGRFieldDefn oField( fieldname.c_str(), fieldType );
   if(fieldType==OFTString)
@@ -196,21 +214,21 @@ void ImgWriterOgr::createField(const string& fieldname, const OGRFieldType& fiel
   if(theLayer<0)
     theLayer=m_datasource->GetLayerCount()-1;//get back layer
   if(m_datasource->GetLayer(theLayer)->CreateField( &oField ) != OGRERR_NONE ){
-      ostringstream es;
+      std::ostringstream es;
       es << "Creating field " << fieldname << " failed";
-      string errorString=es.str();
+      std::string errorString=es.str();
       throw(errorString);
   }
 }
 
-int ImgWriterOgr::getFields(vector<string>& fields, int layer) const
+int ImgWriterOgr::getFields(std::vector<std::string>& fields, int layer) const
 {
   if(layer<0)
     layer=m_datasource->GetLayerCount()-1;
   assert(m_datasource->GetLayerCount()>layer);
   OGRLayer  *poLayer;
   if((poLayer = m_datasource->GetLayer(layer))==NULL){
-    string errorstring="Could not get layer";
+    std::string errorstring="Could not get layer";
     throw(errorstring);
   }
   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
@@ -223,14 +241,14 @@ int ImgWriterOgr::getFields(vector<string>& fields, int layer) const
   return(fields.size());
 }
 
-int ImgWriterOgr::getFields(vector<OGRFieldDefn*>& fields, int layer) const
+int ImgWriterOgr::getFields(std::vector<OGRFieldDefn*>& fields, int layer) const
 {
   if(layer<0)
     layer=m_datasource->GetLayerCount()-1;
   assert(m_datasource->GetLayerCount()>layer);
   OGRLayer  *poLayer;
   if((poLayer = m_datasource->GetLayer(layer))==NULL){
-    string errorstring="Could not get layer";
+    std::string errorstring="Could not get layer";
     throw(errorstring);
   }
   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
@@ -240,7 +258,7 @@ int ImgWriterOgr::getFields(vector<OGRFieldDefn*>& fields, int layer) const
     OGRFieldDefn *poFieldDefn = poFDefn->GetFieldDefn(iField);
     fields[iField]=poFDefn->GetFieldDefn(iField);
   }
-  assert(fields.size()==getFieldCount());
+  assert(fields.size()==getFieldCount(layer));
   return(fields.size());
 }
 
@@ -248,67 +266,67 @@ void ImgWriterOgr::copyFields(const ImgReaderOgr& imgReaderOgr, int theLayer){
   if(theLayer<0)
     theLayer=m_datasource->GetLayerCount()-1;//get back layer
   //get fields from imgReaderOgr
-  vector<OGRFieldDefn*> fields;
+  std::vector<OGRFieldDefn*> fields;
   
-  imgReaderOgr.getFields(fields);
+  imgReaderOgr.getFields(fields,theLayer);
 //   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
   for(int iField=0;iField<fields.size();++iField){
     if(m_datasource->GetLayer(theLayer)->CreateField(fields[iField]) != OGRERR_NONE ){
-      ostringstream es;
+      std::ostringstream es;
       es << "Creating field " << fields[iField]->GetNameRef() << " failed";
-      string errorString=es.str();
+      std::string errorString=es.str();
       throw(errorString);
     }
   }
 }
 
-void ImgWriterOgr::addPoint(double x, double y, const map<string,double>& pointAttributes, string fieldName, const string& theId){
+void ImgWriterOgr::addPoint(double x, double y, const std::map<std::string,double>& pointAttributes, std::string fieldName, const std::string& theId, int layer){
   OGRFeature *poFeature;
-  poFeature=createFeature();
+  poFeature=createFeature(layer);
   OGRPoint pt;
   poFeature->SetField( fieldName.c_str(), theId.c_str());
-  for(map<string,double>::const_iterator mit=pointAttributes.begin();mit!=pointAttributes.end();++mit){
+  for(std::map<std::string,double>::const_iterator mit=pointAttributes.begin();mit!=pointAttributes.end();++mit){
     poFeature->SetField((mit->first).c_str(), mit->second);
   }
   pt.setX(x);
   pt.setY(y);
   poFeature->SetGeometry( &pt );
-  if(createFeature(poFeature)!=OGRERR_NONE){
-    string errorString="Failed to create feature in shapefile";
+  if(createFeature(poFeature,layer)!=OGRERR_NONE){
+    std::string errorString="Failed to create feature in shapefile";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
 }
 
-void ImgWriterOgr::addPoint(double x, double y, const map<string,double>& pointAttributes, string fieldName, int theId){
+void ImgWriterOgr::addPoint(double x, double y, const std::map<std::string,double>& pointAttributes, std::string fieldName, int theId, int layer){
   OGRFeature *poFeature;
-  poFeature = createFeature();
+  poFeature = createFeature(layer);
   OGRPoint pt;
   if(pointAttributes.size()+1!=poFeature->GetFieldCount()){
-    ostringstream ess;
-    ess << "Failed to add feature: " << pointAttributes.size() << " !=" << poFeature->GetFieldCount() << endl;
+    std::ostringstream ess;
+    ess << "Failed to add feature: " << pointAttributes.size() << " !=" << poFeature->GetFieldCount() << std::endl;
     throw(ess.str());
   }
   assert(pointAttributes.size()+1==poFeature->GetFieldCount());
   poFeature->SetField( fieldName.c_str(), theId);
   int fid=0;
-  for(map<string,double>::const_iterator mit=pointAttributes.begin();mit!=pointAttributes.end();++mit){
+  for(std::map<std::string,double>::const_iterator mit=pointAttributes.begin();mit!=pointAttributes.end();++mit){
     poFeature->SetField((mit->first).c_str(),mit->second);
   }
   pt.setX(x);
   pt.setY(y);
   poFeature->SetGeometry( &pt );
-  if(createFeature(poFeature)!=OGRERR_NONE){
-    string errorString="Failed to create feature in shapefile";
+  if(createFeature(poFeature,layer)!=OGRERR_NONE){
+    std::string errorString="Failed to create feature in shapefile";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
 }
 
-//add a line string (polygon), caller is responsible to close the line (end point=start point)
-void ImgWriterOgr::addLineString(vector<OGRPoint*>& points, const string& fieldName, int theId){
+//add a line std::string (polygon), caller is responsible to close the line (end point=start point)
+void ImgWriterOgr::addLineString(std::vector<OGRPoint*>& points, const std::string& fieldName, int theId, int layer){
   OGRFeature *poFeature;
-  poFeature = createFeature();
+  poFeature = createFeature(layer);
   poFeature->SetStyleString("PEN(c:#FF0000,w:5px)");//see also http://www.gdal.org/ogr/ogr_feature_style.html
   poFeature->SetField( fieldName.c_str(), theId);
   OGRLineString theLineString;
@@ -316,20 +334,20 @@ void ImgWriterOgr::addLineString(vector<OGRPoint*>& points, const string& fieldN
   for(int ip=0;ip<points.size();++ip)
     theLineString.setPoint(ip,points[ip]);
   if(poFeature->SetGeometry( &theLineString )!=OGRERR_NONE){
-    string errorString="Failed to set line OGRLineString as feature geometry";
+    std::string errorString="Failed to set line OGRLineString as feature geometry";
     throw(errorString);
   }
-  if(createFeature(poFeature)!=OGRERR_NONE){
-    string errorString="Failed to create feature in shapefile";
+  if(createFeature(poFeature,layer)!=OGRERR_NONE){
+    std::string errorString="Failed to create feature in shapefile";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
 }
 
 //add a ring (polygon), caller is responsible to close the line (end point=start point)?
-void ImgWriterOgr::addRing(vector<OGRPoint*>& points, const string& fieldName, int theId){
+void ImgWriterOgr::addRing(std::vector<OGRPoint*>& points, const std::string& fieldName, int theId, int layer){
   OGRFeature *poFeature;
-  poFeature = createFeature();
+  poFeature = createFeature(layer);
   poFeature->SetStyleString("PEN(c:#FF0000,w:5px)");//see also http://www.gdal.org/ogr/ogr_feature_style.html
   poFeature->SetField( fieldName.c_str(), theId);
   // OGRLineString theLineString;
@@ -343,66 +361,64 @@ void ImgWriterOgr::addRing(vector<OGRPoint*>& points, const string& fieldName, i
   thePolygon.addRing(&theRing);
   // SetSpatialFilter(&thePolygon)
   poFeature->SetGeometry( &thePolygon );
-  if(createFeature(poFeature)!=OGRERR_NONE){
-    string errorString="Failed to create feature in shapefile";
+  if(createFeature(poFeature,layer)!=OGRERR_NONE){
+    std::string errorString="Failed to create feature in shapefile";
     throw(errorString);
     OGRFeature::DestroyFeature( poFeature );
   }
   if(poFeature->SetGeometry( &thePolygon )!=OGRERR_NONE){
-    string errorString="Failed to set polygon as feature geometry";
+    std::string errorString="Failed to set polygon as feature geometry";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
 }
 
 //add a line string (polygon), caller is responsible to close the line (end point=start point)
-void ImgWriterOgr::addLineString(vector<OGRPoint*>& points, const string& fieldName, const string& theId){
+void ImgWriterOgr::addLineString(std::vector<OGRPoint*>& points, const std::string& fieldName, const std::string& theId, int layer){
   OGRFeature *poFeature;
-  poFeature = createFeature();
+  poFeature = createFeature(layer);
   poFeature->SetField( fieldName.c_str(), theId.c_str());
   OGRLineString theLineString;
   theLineString.setNumPoints(points.size());
   for(int ip=0;ip<points.size();++ip)
     theLineString.setPoint(ip,points[ip]);
   if(poFeature->SetGeometry( &theLineString )!=OGRERR_NONE){
-    string errorString="Failed to set line OGRLineString as feature geometry";
+    std::string errorString="Failed to set line OGRLineString as feature geometry";
     throw(errorString);
   }
-  if(createFeature(poFeature)!=OGRERR_NONE){
-    string errorString="Failed to create feature in shapefile";
+  if(createFeature(poFeature,layer)!=OGRERR_NONE){
+    std::string errorString="Failed to create feature in shapefile";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
 }
 
-OGRFeature* ImgWriterOgr::createFeature(){
-  return(OGRFeature::CreateFeature(m_datasource->GetLayer(m_datasource->GetLayerCount()-1)->GetLayerDefn()));
+OGRFeature* ImgWriterOgr::createFeature(int layer){
+  return(OGRFeature::CreateFeature(m_datasource->GetLayer(layer)->GetLayerDefn()));
 }
 
-OGRErr ImgWriterOgr::createFeature(OGRFeature *theFeature){
-  return m_datasource->GetLayer(m_datasource->GetLayerCount()-1)->CreateFeature(theFeature);
+OGRErr ImgWriterOgr::createFeature(OGRFeature *theFeature,int layer){
+  return m_datasource->GetLayer(layer)->CreateFeature(theFeature);
 }
 
 int ImgWriterOgr::getFieldCount(int layer) const
 {
-  if(layer<0)
-    layer=m_datasource->GetLayerCount()-1;
   assert(m_datasource->GetLayerCount()>layer);
   OGRLayer  *poLayer;
   if((poLayer = m_datasource->GetLayer(layer))==NULL){
-    string errorstring="Could not get layer";
+    std::string errorstring="Could not get layer";
     throw(errorstring);
   }
   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
   return(poFDefn->GetFieldCount());
 }
 
-int ImgWriterOgr::getFeatureCount() const
+int ImgWriterOgr::getFeatureCount(int layer) const
 {
-  return(getLayer()->GetFeatureCount());
+  return(getLayer(layer)->GetFeatureCount());
 }
 
-int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, const vector<string>& fieldName, const vector<OGRFieldType>& fieldType, short colX, short colY, const string& theProjection, const OGRwkbGeometryType& eGType, const char fs)
+int ImgWriterOgr::ascii2ogr(const std::string& filename, const std::string &layername, const std::vector<std::string>& fieldName, const std::vector<OGRFieldType>& fieldType, short colX, short colY, const std::string& theProjection, const OGRwkbGeometryType& eGType, const char fs)
 {
   char     **papszOptions=NULL;
   createLayer(layername, theProjection, eGType, papszOptions);
@@ -416,8 +432,8 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
     createField(fieldName[ifield],fieldType[ifield]);
   //create a local OGRFeature, set attributes and attach geometry before trying to write it to the layer. It is imperative that this feature be instantiated from the OGRFeatureDefn associated with the layer it will be written to.
   //todo: try to open and catch if failure...
-  ifstream fpoints(filename.c_str(),ios::in);
-  string line;
+  std::ifstream fpoints(filename.c_str(),std::ios::in);
+  std::string line;
   OGRPolygon thePolygon;
   OGRLinearRing theRing;
   OGRPoint firstPoint;
@@ -427,15 +443,15 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
 
 
   if(fs>' '&&fs<='~'){//field separator is a regular character (minimum ASCII code is space, maximum ASCII code is tilde)
-    string csvRecord;
+    std::string csvRecord;
     while(getline(fpoints,csvRecord)){//read a line
       OGRFeature *pointFeature;
       if(eGType==wkbPoint)
         pointFeature=createFeature();
       OGRPoint thePoint;
       bool skip=false;
-      istringstream csvstream(csvRecord);
-      string value;
+      std::istringstream csvstream(csvRecord);
+      std::string value;
       int colId=0;
       int fieldId=0;
       while(getline(csvstream,value,fs)){//read a column
@@ -471,14 +487,14 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
         ++colId;
       }
       if(colId!=fieldId+2){
-        ostringstream ess;
+        std::ostringstream ess;
         ess << "Error: colId = " << colId << " is different from fieldId+2 = " << fieldId;
         throw(ess.str());
       }
       if(eGType==wkbPoint){
         pointFeature->SetGeometry( &thePoint );
         if(createFeature(pointFeature)!=OGRERR_NONE){
-          string errorString="Failed to create feature in shapefile";
+          std::string errorString="Failed to create feature in shapefile";
           throw(errorString);
           OGRFeature::DestroyFeature( pointFeature );
         }
@@ -498,8 +514,8 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
         pointFeature=createFeature();
       OGRPoint thePoint;
       bool skip=false;
-      istringstream ist(line);
-      string value;
+      std::istringstream ist(line);
+      std::string value;
       int colId=0;
       int fieldId=0;
       while(ist >> value){
@@ -535,14 +551,14 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
         ++colId;
       }
       if(colId!=fieldId+2){
-        ostringstream ess;
+        std::ostringstream ess;
         ess << "Error: colId = " << colId << " is different from fieldId+2 = " << fieldId;
         throw(ess.str());
       }
       if(eGType==wkbPoint){
         pointFeature->SetGeometry( &thePoint );
         if(createFeature(pointFeature)!=OGRERR_NONE){
-          string errorString="Failed to create feature in shapefile";
+          std::string errorString="Failed to create feature in shapefile";
           throw(errorString);
           OGRFeature::DestroyFeature( pointFeature );
         }
@@ -561,7 +577,7 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
     // SetSpatialFilter(&thePolygon)
     polyFeature->SetGeometry( &thePolygon );
     if(createFeature(polyFeature)!=OGRERR_NONE){
-      string errorString="Failed to create feature in shapefile";
+      std::string errorString="Failed to create feature in shapefile";
       throw(errorString);
       OGRFeature::DestroyFeature( polyFeature );
     }
@@ -572,28 +588,26 @@ int ImgWriterOgr::ascii2ogr(const string& filename, const string &layername, con
 int ImgWriterOgr::addData(const ImgReaderGdal& imgReader, int theLayer, bool verbose)
 {
   OGRLayer  *poLayer;
-  if(theLayer<0)
-    theLayer=m_datasource->GetLayerCount()-1;//get back layer
   assert(m_datasource->GetLayerCount()>theLayer);
   if(verbose)
-    cout << "number of layers: " << m_datasource->GetLayerCount() << endl;
+    std::cout << "number of layers: " << m_datasource->GetLayerCount() << std::endl;
   if(verbose)
-    cout << "get layer " << theLayer << endl;
+    std::cout << "get layer " << theLayer << std::endl;
   poLayer = m_datasource->GetLayer(theLayer);
   //start reading features from the layer
   OGRFeature *poFeature;
   if(verbose)
-    cout << "reset reading" << endl;
+    std::cout << "reset reading" << std::endl;
   poLayer->ResetReading();
   for(int iband=0;iband<imgReader.nrOfBand();++iband){
-    ostringstream fs;
+    std::ostringstream fs;
     fs << "band" << iband;
     createField(fs.str(),OFTReal);
   }
   OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
   int nfield=poFDefn->GetFieldCount();
   if(verbose)
-    cout << "new number of fields: " << nfield << endl;
+    std::cout << "new number of fields: " << nfield << std::endl;
   while( (poFeature = poLayer->GetNextFeature()) != NULL ){
     OGRGeometry *poGeometry;
     poGeometry = poFeature->GetGeometryRef();
@@ -606,7 +620,7 @@ int ImgWriterOgr::addData(const ImgReaderGdal& imgReader, int theLayer, bool ver
       double imgData;
       imgReader.readData(imgData,GDT_Float64,x,y,iband);
       //todo: put imgdata in field
-      ostringstream fs;
+      std::ostringstream fs;
       fs << "band" << iband;
       poFeature->SetField(fs.str().c_str(),imgData);
     }
